/**
*	@file: TGATexture.cpp
*		@version	1.0	
*		@date		2008/09/27
*
*	@brief TGAeNX`̓ǂݍ
*
*	@author Muraki Tatunori 
**/

#include "TGATexture.h"
#include <GL/glut.h>
#include <fstream>
#include <iostream>
#include <vector>

//eNX`Ǘhc
std::map< unsigned int, unsigned int > TGATexture::m_Textures;

//eNX`̓ǂݍ
bool TGATexture::load( unsigned int id, const std::string& filename )
{
	unsigned char header[18]; 
	unsigned char bytePerPixel;
	unsigned int temp;

	std::ifstream textureFile( filename.c_str(), std::ios::in | std::ios::binary );

	//@t@CJ
	if( textureFile.fail() ){
		std::cout << "Error : w肵t@CJ܂ł\n";
		std::cout << "File Name : " << filename << std::endl;

		return false;
	}

	//@wb_[̓ǂݍ
	textureFile.read( (char*)header, sizeof(header) );
	
	unsigned int width;
	unsigned int height;
	//@ƍ߂
	width = header[13] * 256 + header[12];
	height = header[15] * 256 + header[14];
	
	//@rbg̐[
	unsigned int bpp = header[16];
	unsigned int format;
	unsigned int internalFormat;

	//@24 bit
	if ( bpp == 24 )
	{
		format = GL_RGB;
		internalFormat = GL_RGB;
	}
	//@32 bit
	else if ( bpp = 32 )
	{
		format = GL_RGBA;
		internalFormat = GL_RGBA;
	}

	//@1sNZ̃oCg
	bytePerPixel = bpp/8;

	//@f[^TCY̌
	unsigned int imageSize;
	imageSize = width * height * bytePerPixel;

	//@m
	std::vector< unsigned char > imageData;
	imageData.resize( imageSize );

	if( header[2] == 2 ){
		//@eNZf[^Cɓǂݎ
		textureFile.read( (char*)&imageData[0], imageSize );

		//@BGR(A)RGB(A)ɃRo[g
		for ( unsigned int i=0; i < imageSize; i += bytePerPixel )
		{
			temp = imageData[i];
			imageData[i+0] = imageData[i+2];
			imageData[i+2] = temp;
		}
	}

	if( header[2] == 10 ){
		unsigned int pixel = 0; //݂̃sNZ
		unsigned int byte = 0; //ڂĂf[^̃oCg

		//obt@̐
		std::vector< unsigned char > buffer( bytePerPixel );

		unsigned char chunk = 0; //`Nwb_[
		while( byte < imageSize ){
			//`Nwb_[̓ǂݍ
			textureFile.read( (char*)&chunk, sizeof(chunk) );

			//f[^̓ǂݍ
			if( chunk < 128 ){
				chunk += 1;

				for( int i = 0; i < chunk; i++ ){
					//f[^̓ǂݍ
					textureFile.read( (char*)&buffer[0], bytePerPixel );

					//sNZf[^̐ݒ
					imageData[byte] = buffer[2];
					imageData[byte+1] = buffer[1];
					imageData[byte+2] = buffer[0];
					if( bytePerPixel == 4 ){
						imageData[byte+3] = buffer[3];
					}

					//̃sNZ
					byte += bytePerPixel;

					//ǂݍ񂾃sNZJEg
					pixel += 1;
				}
				continue;
			}

			//RLEk̓ǂݍ
			chunk -= 127;
			//f[^̓ǂݍ
			textureFile.read( (char*)&buffer[0], bytePerPixel );
			
			for( int i = 0; i < chunk; i++ ){
				//sNZf[^̐ݒ
				imageData[byte] = buffer[2];
				imageData[byte+1] = buffer[1];
				imageData[byte+2] = buffer[0];
				if( bytePerPixel == 4 ){
					imageData[byte+3] = buffer[3];
				}

				//̃sNZ
				byte += bytePerPixel;

				//ǂݍ񂾃sNZJEg
				pixel += 1;
			}
		}
	}

	//eNX`̐
	unsigned int textureId;
	glGenTextures( 1, &textureId );

	//eNX`̃oCh
	glBindTexture( GL_TEXTURE_2D, textureId );

	if( bpp == 24 ){
		glPixelStorei( GL_UNPACK_ALIGNMENT, 4 );
	}else if( bpp == 32 ){
		glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
	}

	//eNX`̊蓖
	gluBuild2DMipmaps( 
		GL_TEXTURE_2D,
		internalFormat,
		width,
		height,
		format,
		GL_UNSIGNED_BYTE,
		&imageData[0]
	);

	//glTexImage2D( 
	//	GL_TEXTURE_2D,
	//	0,
	//	internalFormat,
	//	width,
	//	height,
	//	0,
	//	format,
	//	GL_UNSIGNED_BYTE,
	//	imageData
	//);

    //eNX`gEk@̎w
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

    //eNX`
    glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

	m_Textures[id] = textureId;

	glBindTexture( GL_TEXTURE_2D, 0 );
	glDisable( GL_TEXTURE_2D );

	return true;
}

//eNX`̃oCh
void TGATexture::bind( unsigned int id )
{
	glEnable( GL_TEXTURE_2D );
	glBindTexture( GL_TEXTURE_2D, m_Textures[id] );
}

//eNX`̊J
void TGATexture::release(unsigned int id)
{
	glDeleteTextures( 1, &m_Textures[id] );

	m_Textures[id] = 0;
}

//eNX`̎擾
unsigned int TGATexture::getID(unsigned int id)
{
	return m_Textures[id];
}

/*	End of File	*/
